@@ -1,6 +1,6 @@
 /*
  * GNUsound - a sound editor for GNOME.
- * Copyright (C) 2002-2004  Pascal Haakmat <a.haakmat@chello.nl>
+ * Copyright (C) 2002-2005  Pascal Haakmat <pascal@gnu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -132,7 +132,6 @@ alsa_handle_errors(struct player *p,
                  snd_pcm_name(handle), snd_strerror(*err));
             return *err;
         }
-        player_set_position(p, pos);
         *err = 0;
         return 0;
     case -EAGAIN:
@@ -151,33 +150,15 @@ alsa_play(struct player *p,
     struct alsa_data *dd = p->driver_data;
     int bytes_per_frame = sample_get_width(p->state->sample_type) * 
         p->shl->clip->mixer->output_channels;
-    snd_pcm_sframes_t err = 0, avail;
+    snd_pcm_sframes_t err = 0, avail = player_get_chunk_size(p);
     void *buf;
+    
+    /* Must transfer on period_size boundaries. */
 
-    /*
-     * Find out how many frames the audio device can accept and fetch
-     * at most that amount to play back.
-     */
+    if(frame_count >= dd->period_size) 
+        frame_count = (MIN(frame_count, avail) / dd->period_size) * 
+            dd->period_size;
 
-    if((avail = snd_pcm_avail_update(dd->play_handle)) < 0) {
-        FAIL("snd_pcm_avail_update: %s\n", snd_strerror(avail));
-        if(alsa_handle_errors(p,
-                              dd->play_handle, 
-                              p->state->playback_pos,
-                              &avail, 
-                              pref_get_as_int("alsa.underrun_recovery"),
-                              1) < 0) 
-            return avail;
-
-        /*
-         * If the error was not serious or recoverable then return
-         * success and hope for the best on the next iteration.
-         */
-
-        return 0;
-    }
-
-    frame_count = MIN(frame_count, avail);
     if((err = player_get_playback_bufi(p, &buf, &frame_count))) {
         FAIL("error getting playback buffer\n");
         return err;
@@ -201,7 +182,7 @@ alsa_play(struct player *p,
 
     //    DEBUG("writing %ld frames...\n", frame_count);
     err = snd_pcm_writei(dd->play_handle, buf, frame_count);
-    //    DEBUG("...returned from write, result: %ld\n", err);
+    //    DEBUG("...returned from write, result: %ld (%s)\n", err, snd_strerror(err));
     if(alsa_handle_errors(p,
                           dd->play_handle, 
                           p->state->playback_pos,
@@ -210,8 +191,6 @@ alsa_play(struct player *p,
                           1) < 0) 
         return err;
     
-    snd_pcm_delay(dd->play_handle, &dd->delay);
-
     if((err = player_flush_playback_bufi(p, err))) {
         FAIL("error flushing playback buffer\n");
         return err;
@@ -250,7 +229,7 @@ alsa_record(struct player *p,
         FAIL("error flushing record buffer\n");
         return err;
     }
-    
+
     return 0;
 }
 
@@ -258,21 +237,28 @@ int
 alsa_transfer(struct player *p) {
     struct alsa_data *dd = p->driver_data;
     AFframecount err;
-    int started = 0;
-
+    int r, first = 1;
+    snd_pcm_uframes_t f;
+    snd_pcm_status_t *status;
+    int nfds_play = snd_pcm_poll_descriptors_count(dd->play_handle), 
+        nfds_record = (p->state->record_mode ? 
+                       snd_pcm_poll_descriptors_count(dd->record_handle) : 0);
+    struct pollfd pollfds[nfds_play + nfds_record];
+    
+    snd_pcm_status_alloca(&status);
+    
     while(player_has_work(p)) {
+
         if((err = alsa_play(p, player_get_playback_avail(p))) < 0) {
             FAIL("playback error, err: %ld\n", err);
             view_set_transient(p->shl->view, MSG_ERR, "Playback error %d", err);
             return err;
         }
 
-        /*
-        if(!started) {
+        if(first) {
             snd_pcm_start(dd->play_handle);
-            started = 1;
+            first = 0;
         }
-        */
 
         if(p->state->record_mode) {
             if((err = alsa_record(p, player_get_record_avail(p))) < 0) {
@@ -280,15 +266,33 @@ alsa_transfer(struct player *p) {
                 view_set_transient(p->shl->view, MSG_ERR, "Record error %d", err);
                 return err;
             }
-        }
-        //        snd_pcm_delay(dd->play_handle, &dd->delay);
 
-        snd_pcm_wait(dd->play_handle, 1000);
+            snd_pcm_status(dd->record_handle, status);
+            if((f = snd_pcm_status_get_overrange(status))) 
+                view_set_transient(p->shl->view, MSG_WARN, "Overrange %ld", f);
+            
+        }
+        
+        /* Set up structures for poll(). */
+
+        nfds_play = snd_pcm_poll_descriptors(dd->play_handle,
+                                             pollfds, 
+                                             nfds_play);
+        if(p->state->record_mode) 
+            nfds_record = snd_pcm_poll_descriptors(dd->record_handle, 
+                                                   &pollfds[nfds_play], 
+                                                   nfds_record);
+
+        /* Wait for activity. */
+
+        r = poll(pollfds, nfds_play + nfds_record, 500);
+
     }
 
     /* Make sure final samples are played. */
 
     snd_pcm_nonblock(dd->play_handle, 0);
+
     return dd->err;
 }
 
@@ -334,7 +338,7 @@ set_swparams(struct player *p,
     
     err = snd_pcm_sw_params_set_start_threshold(handle,
                                                 swparams,
-                                                handle==dd->record_handle ? 0 : dd->buffer_size);
+                                                (dd->buffer_size / dd->period_size) * dd->period_size);
     if (err < 0) {
         FAIL("Unable to set start threshold mode for %s: %s\n",
              snd_pcm_name(handle),
@@ -409,7 +413,7 @@ set_hwparams(struct player *p,
     int err, dir = -1;
     unsigned int rrate;    
     unsigned int buffer_time = p->state->audio_chunk_duration * 1000000;
-    unsigned int period_time = 50000;
+    unsigned int period_time = buffer_time / 20;
     struct alsa_data *dd = p->driver_data;
 
     DEBUG("format: %d, channels: %d, rate: %d, access: %d\n",
@@ -488,6 +492,11 @@ set_hwparams(struct player *p,
              snd_strerror(err));
         return err;
     }
+    /* FIXME: possibly dangerous nonsense since playback and record
+       buffers can be different size. Somewhat mitigated by the fact
+       that record buffers are generally smaller and the record device
+       is initialized after the playback device. */
+
     dd->playback_buffer_size = dd->buffer_size;
     dd->record_buffer_size = dd->buffer_size;
     DEBUG("buffer size: %ld\n", dd->buffer_size);
@@ -534,13 +543,14 @@ alsa_init_device(struct player *p,
                  int format,
                  int channels,
                  int rate,
-                 snd_pcm_stream_t stream) {
+                 snd_pcm_stream_t stream,
+                 int mode) {
     struct alsa_data *dd = p->driver_data;
     int err;
 
     *handle = NULL;
 
-    if((err = snd_pcm_open(handle, dev, stream, SND_PCM_NONBLOCK)) < 0) {
+    if((err = snd_pcm_open(handle, dev, stream, mode)) < 0) {
         FAIL("Device %s open error: %s\n", dev, snd_strerror(err));
         view_set_transient(p->shl->view, MSG_ERR, "Can't open %s", dev);
         return err;
@@ -626,7 +636,8 @@ alsa_open(struct player *p) {
                                format,
                                p->shl->clip->mixer->output_channels,
                                p->state->source_rate,
-                               SND_PCM_STREAM_PLAYBACK)) < 0) {
+                               SND_PCM_STREAM_PLAYBACK,
+                               SND_PCM_NONBLOCK)) < 0) {
         FAIL("Initialization error on playback device %s: %s\n", 
              pref_get_as_string("alsa.playback_device"),
              snd_strerror(err));
@@ -644,7 +655,8 @@ alsa_open(struct player *p) {
                                    format,
                                    p->state->target_tracks,
                                    p->state->source_rate,
-                                   SND_PCM_STREAM_CAPTURE)) < 0) {
+                                   SND_PCM_STREAM_CAPTURE,
+                                   SND_PCM_NONBLOCK)) < 0) {
             FAIL("Initialization error on record device %s: %s\n", 
                  pref_get_as_string("alsa.record_device"),
                  snd_strerror(err));
