@@ -1,6 +1,6 @@
 /*
  * GNUsound - a sound editor for GNOME.
- * Copyright (C) 2002-2004  Pascal Haakmat <a.haakmat@chello.nl>
+ * Copyright (C) 2002-2004  Pascal Haakmat <pascal@gnu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,9 @@
 
 #else /* HAVE_JACK */
 
+#undef GTK_DISABLE_DEPRECATED
+#undef GNOME_DISABLE_DEPRECATED
+
 #include <gnusound.h>
 #include <jack/jack.h>
 #include <signal.h>
@@ -71,23 +74,94 @@ struct jackdrv_data {
 static int self_id;
 static struct jackdrv_globals jackdrv_globals = { 0, };
 
-const char **
+void
+jackdrv_free_ports(char **ports) {
+    int i;
+
+    if(!ports)
+        return;
+
+    for(i = 0; ports[i]; i++) 
+        mem_free(ports[i]);
+    
+    mem_free(ports);
+}
+
+/**
+ * Gets a list of ports from JACK which satisfy the given patterns and
+ * flags.  To do this the function briefly establishes a connection,
+ * gets the ports, then drops the connection again.
+ * @param port_name_pattern A port name pattern that a port needs to satisfy.
+ * @param type_name_pattern A type name pattern that a port needs to satisfy.
+ * @param flags The flags which a port needs to satisfy.
+ * @return NULL terminated array of port name strings. This array must
+ * be freed using jackdrv_free_ports().
+ */
+
+char **
 jackdrv_get_ports(const char *port_name_pattern,
                   const char *type_name_pattern,
                   unsigned long flags) {
     jack_client_t *client;
-    const char **ports;
+    const char **ports_from_jack;
+    char **ports;
+    int i, nports;
+
     if((client = jack_client_new("jackdrv")) == 0) {
         FAIL("jack server not running?\n");
         return NULL;
     }
 
-    ports = jack_get_ports(client,
-                           port_name_pattern,
-                           type_name_pattern,
-                           flags);
+    ports_from_jack = jack_get_ports(client,
+                                     port_name_pattern,
+                                     type_name_pattern,
+                                     flags);
+
+    DEBUG("port_name_pattern: %s, type_name_pattern: %s, flags: %u\n",
+          port_name_pattern, type_name_pattern, flags);
+
+    if(!ports_from_jack) {
+        FAIL("could not get ports from jack\n");
+        goto err_ports;
+    }
+
+    /* Count ports returned. */
+
+    for(nports = 0; ports_from_jack[nports]; nports++);
+
+    /* Allocate array to copy port names returned by JACK. We need to
+       copy the port names since JACK will free them once we close the
+       client connection to JACK. */
+
+    ports = mem_alloc(sizeof(char *) * (nports + 1));
+
+    if(!ports) {
+        FAIL("could not copy ports from jack\n");
+        goto err_alloc;
+    }
+
+    for(i = 0; i < nports; i++) {
+        ports[i] = mem_alloc(strlen(ports_from_jack[i]) + 1);
+        if(!ports[i]) {
+            for(; i >= 0; i--)
+                mem_free(ports[i]);
+            goto err_alloc;
+        }
+        strcpy(ports[i], ports_from_jack[i]);
+    }
+    ports[nports] = NULL;
+
+    free(ports_from_jack);
     jack_client_close(client);
+
     return ports;
+
+ err_alloc:
+    free(ports_from_jack);
+ err_ports:
+    jack_client_close(client);
+
+    return NULL;
 }
 
 size_t 
@@ -182,6 +256,7 @@ void
 jackdrv_shutdown(void *arg) {
     DEBUG("jack shutting down\n");
     jackdrv_globals.connected = 0;
+    jackdrv_globals.client = NULL;
 }
 
 int
@@ -595,22 +670,25 @@ jackdrv_new() {
 static struct pane *config_pane = NULL;
 static GHashTable *jackdrv_input_channel_selections;
 static GHashTable *jackdrv_output_channel_selections;
-static const char **jackdrv_input_ports;
-static const char **jackdrv_output_ports;
+static char **jackdrv_input_ports;
+static char **jackdrv_output_ports;
 
 
 GtkWidget * 
 jackdrv_make_ports_menu(const char *selected_port,
-                        const char **ports) {
+                        char **ports) {
     int i, match = -1;
     GtkWidget *menu;
     GList *l = NULL;
     
+    DEBUG("selected_port: %s, ports: %p\n", selected_port, ports);
+
     l = g_list_append(l, "unspecified");
     for(i = 0; ports && ports[i]; i++) {
+        DEBUG("ports[%d]: %s\n", i, ports[i]);
         if(selected_port && !strcmp(ports[i], selected_port))
             match = i;
-        l = g_list_append(l, (char *)ports[i]);
+        l = g_list_append(l, ports[i]);
     }
     
     menu = combo_box_new();
@@ -631,7 +709,7 @@ jackdrv_map(int port_count,
             const char *port_template,
             GHashTable *ht,
             GtkWidget *map_table,
-            const char **ports) {
+            char **ports) {
     int i;
     char s[100];
     char path[255], *selected_port;
@@ -686,6 +764,9 @@ jackdrv_populate_gui() {
                 jackdrv_input_channel_selections,
                 pane_get_widget(config_pane, "input_channel_map"),
                 jackdrv_output_ports);
+
+    jackdrv_free_ports(jackdrv_input_ports);
+    jackdrv_free_ports(jackdrv_output_ports);
     
 }
 
@@ -733,14 +814,17 @@ void
 jackdrv_commit_channels(gpointer key,
                         gpointer value,
                         gpointer user_data) {
-    const char *port = combo_box_get_value(COMBO_BOX(value));;
+    const char *port = combo_box_get_value(COMBO_BOX(value));
     char path[256];
 
+    snprintf(path, sizeof(path), "/gnusound/preferences/jack.%s", 
+             (char *)key);
+
     if(strcmp(port, "unspecified")) {
-        snprintf(path, sizeof(path), "/gnusound/preferences/jack.%s", 
-                 (char *)key);
         gnome_config_set_string(path, port);
-    }    
+    } else {
+        gnome_config_clean_key(path);
+    }
 }
 
 void
@@ -798,7 +882,7 @@ jackdrv_commit_config() {
                  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pane_get_widget(config_pane, "input_channels"))));
     pref_set_int("jack.buffers",
                  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pane_get_widget(config_pane, "buffers"))));
-    
+
     g_hash_table_foreach(jackdrv_output_channel_selections,
                          jackdrv_commit_channels,
                          jackdrv_input_ports);
