@@ -1,6 +1,6 @@
 /*
  * GNUsound - a sound editor for GNOME.
- * Copyright (C) 2002-2004  Pascal Haakmat <a.haakmat@chello.nl>
+ * Copyright (C) 2002-2004  Pascal Haakmat <pascal@gnu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,9 @@
  *
  */
 
+#undef GTK_DISABLE_DEPRECATED
+#undef GNOME_DISABLE_DEPRECATED
+
 #include <config.h>
 #include <math.h>
 #include <errno.h>
@@ -45,7 +48,7 @@ mixer_is_source_solo(mixer *m,
              source_channel);
         return 0;
     }
-    return m->source_is_solo[source_channel] ? 1 : 0;
+    return m->source_is_solo[source_channel];
 }
 
 /**
@@ -59,7 +62,7 @@ mixer_is_source_mute(mixer *m,
              source_channel);
         return 0;
     }
-    return m->source_is_mute[source_channel] ? 0 : 1;
+    return m->source_is_mute[source_channel];
 }
 
 /**
@@ -75,7 +78,7 @@ mixer_toggle_source_solo(mixer *m,
         return;
     }
     
-    m->source_is_solo[source_channel] = ~m->source_is_solo[source_channel];
+    m->source_is_solo[source_channel] = !m->source_is_solo[source_channel];
     DEBUG("source_channel: %d, new solo mask: %x\n", 
           source_channel, m->source_is_solo[source_channel]);
     m->num_solos += (m->source_is_solo[source_channel] ? 1 : -1);
@@ -94,7 +97,7 @@ mixer_toggle_source_mute(mixer *m,
     }
     DEBUG("source_channel: %d, new mute mask: %x\n", 
           source_channel, m->source_is_mute[source_channel]);
-    m->source_is_mute[source_channel] = ~m->source_is_mute[source_channel];
+    m->source_is_mute[source_channel] = !m->source_is_mute[source_channel];
 }
 
 /**
@@ -332,15 +335,17 @@ mixer_mixi(mixer *m,
 
 #define MIXN(type) \
   for(j = 0; j < m->output_channels; j++) { \
-    for(k = 0; k < m->source_tracks; k++)  \
+    for(k = 0; k < m->source_tracks; k++) { \
+        if(m->num_solos > 0 && !m->source_is_solo[k]) { \
+            continue; \
+        } else if(m->source_is_mute[k]) { \
+            continue; \
+        } \
         for(i = 0; i < frame_count; i++) { \
             ((type **)fb_targets)[j][i] += \
-                ((int)((type **)fb_sources)[k][i] & \
-                 (m->num_solos ? \
-                  m->source_is_solo[k] : \
-                  m->source_is_mute[k])) * \
-                m->mixtable[j][k]; \
-      } \
+                ((type **)fb_sources)[k][i] * m->mixtable[j][k]; \
+        } \
+    } \
   } \
 
 void
@@ -417,7 +422,7 @@ mixer_insert_source_tracks(mixer *m,
                     (max_tracks - i - 1) * sizeof(m->source_is_mute[i]));
 
             m->source_is_solo[i] = 0;
-            m->source_is_mute[i] = 0xFFFFFFFF;
+            m->source_is_mute[i] = 0;
 
             if(src && i < src->source_tracks) {
                 m->source_is_solo[i] = src->source_is_solo[k];
@@ -812,7 +817,7 @@ mixer_new(int output_channels,
     m->source_is_solo = &ptr[MAX_TRACKS];
     for(i = 0; i < MAX_TRACKS; i++) {
         /* Unmute all source channels. */
-        m->source_is_mute[i] = 0xFFFFFFFF;
+        m->source_is_mute[i] = 0;
         /* Unsolo all source channels. */
         m->source_is_solo[i] = 0;
     }
@@ -958,16 +963,13 @@ mixer_load(mixer *m,
         g_free(argv);
     }
     
-    /* "Source Is Mute" settings are stored on disk basically opposite
-       from how they are stored in memory: a 1 on disk means a 0 in
-       memory and a 0 on disk means 0xFFFFFFFF in memory. Reason: to
-       make it easier for readers of the config file to understand. */
+    /* Recover mute settings. */
 
     snprintf(tmp, 512, "=%s=/Mixer/Source Is Mute", path);
     gnome_config_get_vector(tmp, &argc, &argv);
     for(j = 0; j < argc; j++) {
         if(j < m->source_tracks) 
-            m->source_is_mute[j] = (atoi(argv[j]) ? 0 : 0xFFFFFFFF);
+            m->source_is_mute[j] = (atoi(argv[j]) ? 1 : 0);
         else {
             r = 1;
             FAIL("is muted channel %d (value: %d) out of bounds.\n",
@@ -979,14 +981,13 @@ mixer_load(mixer *m,
         g_free(argv[j]);
     g_free(argv);
 
-    /* For "Source Is Solo" it's the same story as for "Source Is
-       Mute" above. */
+    /* Recover solo settings. */
 
     snprintf(tmp, 512, "=%s=/Mixer/Source Is Solo", path);
     gnome_config_get_vector(tmp, &argc, &argv);
     for(j = 0; j < argc; j++) {
         if(j < m->source_tracks) 
-            m->source_is_solo[j] = (atoi(argv[j]) ? 0xFFFFFFFF : 0);
+            m->source_is_solo[j] = (atoi(argv[j]) ? 1 : 0);
         else {
             r = 1;
             FAIL("is muted channel %d (value: %d) out of bounds.\n",
